@@ -15,13 +15,6 @@
 #include <Rinternals.h>
 #include <R_ext/GraphicsEngine.h>
 
-/* This is so that cairoDevice builds under R 2.7 and 2.9
-   simultaneously. Remove after R 2.8 is released. */
-#include <Rversion.h>
-#if R_VERSION >= R_Version(2,8,0)
-# define NewDevDesc DevDesc
-#endif
-
 typedef struct _Cairo_locator_info {
     guint x;
     guint y;
@@ -30,22 +23,29 @@ typedef struct _Cairo_locator_info {
     gboolean active;
 } CairoLocator;
 
+#if R_GE_version < 7
 typedef struct _CairoEvent {
   SEXP rho;
   SEXP result;
   gboolean active;
 } CairoEvent;
+#endif
 
 typedef struct {
-	GtkWidget *window;			/* Graphics frame */
-	GtkWidget *drawing;			/* widget to which we are drawing */
-	GdkPixmap *pixmap;			/* off-screen drawable */
-	cairo_t *cr;				/* the cairo context to which we draw */
+  GtkWidget *window;			/* Graphics frame */
+  GtkWidget *drawing;			/* widget to which we are drawing */
+  GdkPixmap *pixmap;			/* off-screen drawable */
+  cairo_t *cr;				/* the cairo context
+                                           to which we draw */
+  cairo_t *cr_custom; /* custom cairo context from R */
   cairo_surface_t *surface; /* if non-NULL we have an alt surface like svg */
   gchar *filename; /* filename for certain Cairo backends */
 	gint width, height; /* width and height of device */
+#if R_GE_version < 7
   CairoEvent *event; /* stores information for 'getGraphicsEvent' support */
+#endif
   CairoLocator *locator; /* stores information for 'locator' support */
+  int holdlevel; /* apparently we have to track the hold level */
 } CairoDesc;
 
 /* Device driver actions */
@@ -70,6 +70,14 @@ static void Cairo_NewPage(const pGEcontext gc, pDevDesc dd);
 static void Cairo_Polygon(int n, double *x, double *y, 
 			const pGEcontext gc,
 			pDevDesc dd);
+static SEXP Cairo_Cap(pDevDesc dd);
+static void Cairo_Raster(unsigned int *raster, int w, int h,
+                         double x, double y, 
+                         double width, double height,
+                         double rot, 
+                         Rboolean interpolate,
+                         const pGEcontext gc, pDevDesc dd);
+
 static void Cairo_Polyline(int n, double *x, double *y, 
 			 const pGEcontext gc,
 			 pDevDesc dd);
